function fitCubeModel(file, outfile)
% Function for fitting model to reconstruction

if exist(file)
rec = importdata(file);
if isstruct(rec)
rec = rec.reconstruction;
end
rec = padarray(rec,[10 10 10]);
count = 1;
arrayScaleFactor = 5;
bestR = 1e30;
arraySize = size(rec,1)*arrayScaleFactor;

cubeSize_vec = (arrayScaleFactor*23):(24*arrayScaleFactor);
shellThickness_vec = (arrayScaleFactor*0):1:ceil((9*arrayScaleFactor/5));
shellIntensity_vec = [46];
num_params = numel(cubeSize_vec) * numel(shellThickness_vec) * numel(shellIntensity_vec)
all_cube_sizes = zeros(1, num_params);
all_shell_thicknesses = zeros(1, num_params);
all_Rs = zeros(1, num_params);

for cubeSize = cubeSize_vec;
cubeSize
for shellThickness = shellThickness_vec;
shellThickness
for shellIntensity = shellIntensity_vec;
% tic
coreIntensity = 79;


nc = size(rec,1)/2+1;
flag = 0;
model = makeCoreShellModel2(arraySize,cubeSize,shellThickness,shellIntensity,coreIntensity);
model = convn(model,ones(arrayScaleFactor,arrayScaleFactor,arrayScaleFactor)./(arrayScaleFactor^3),'same');
model = model(1:arrayScaleFactor:end,1:arrayScaleFactor:end,1:arrayScaleFactor:end);

if mod(cubeSize,2)==0
nn = size(model,1);
m = zeros(nn+1,nn+1,nn+1);
m(1:nn,1:nn,1:nn) = model;
m = max(m, m(end:-1:1,end:-1:1,end:-1:1));
model = m(1:end-1,1:end-1,1:end-1);

end
shiftMax = 2;
scaleFactor = mean(mean(mean(rec(nc-5:nc+5,nc-5:nc+5,nc-5:nc+5))))./mean(mean(mean(model(nc-5:nc+5,nc-5:nc+5,nc-5:nc+5))));
model = model .* scaleFactor;

[xcorr, shiftX, shiftY, shiftZ] = CrossCorrelate_v2(rec,model);
model = circshift(model,[shiftX, shiftY, shiftZ]);
Rs(count) = sum(sum(sum(abs(rec - model))))./sum(abs(model(:)));

all_cube_sizes(count) = cubeSize;
all_shell_thicknesses(count) = shellThickness;
all_Rs(count) = Rs(count);
if Rs(count) < bestR
bestR = Rs(count)
bestModel_R = model;
bestModel_pars_R.shellThickness = shellThickness;
bestModel_pars_R.cubeSize = cubeSize;
bestModel_pars_R.shellIntensity = shellIntensity;
bestModel_pars_R.coreSize = cubeSize - shellThickness*2;
bestModel_pars_R.shellIntensity = shellIntensity;
end

count = count+1;

end
end
end
save(outfile,'bestR','bestModel_R','bestModel_pars_R', 'all_cube_sizes', 'all_shell_thicknesses', 'all_Rs')
end
end

